Option Strict On
Imports System
Public Class FindNextPrimeNumber

'  
Public Enum ProcessingState
  notYetStarted
  waitingToStartAsync
  lookingForPrime
  foundPrime
  requestAbort
  aborted
End Enum

Private m_startTickCount As Integer
Private m_endTickCount As Integer
Private m_startPoint As Long
Private m_NextHighestPrime As Long
Private m_processingState As ProcessingState

'--------------------------------------------------
'  
'--------------------------------------------------
Public Sub setProcessingState(ByVal nextState _
                                As ProcessingState)
  '------------------------------------------------------------
  '  , 
  '       
  '      
  '------------------------------------------------------------
  Dim currentState As ProcessingState
  currentState = getProcessingState()
  If ((currentState = ProcessingState.aborted) _
    OrElse (currentState = ProcessingState.foundPrime)) Then
    Return
  End If

  '   
  SyncLock (Me)
    '  
    m_processingState = nextState
  End SyncLock
End Sub

Public Function getProcessingState() As ProcessingState
  Dim currentState As ProcessingState
  '   
  SyncLock (Me)
    currentState = m_processingState
  End SyncLock

  Return currentState
End Function

Public Function getTickCountDelta() As Integer
  If (getProcessingState() = _
          ProcessingState.lookingForPrime) Then
    Throw New Exception( _
     "   !     ")
  End If

  Return m_endTickCount - m_startTickCount
End Function

'---------------------------------------------------
'  
'---------------------------------------------------
Public Function getPrime() As Long
  If (getProcessingState() <> ProcessingState.foundPrime) Then
    Throw New Exception("    !")
  End If

  Return m_NextHighestPrime
End Function

' 
Public Sub New(ByVal startPoint As Long)
  setProcessingState(ProcessingState.notYetStarted)
  m_startPoint = startPoint
End Sub

'-----------------------------------------------------------
'   ,    
'findNextHighestPrime()
'-----------------------------------------------------------
Public Sub findNextHighestPrime_Async()
  Dim threadStart As System.Threading.ThreadStart
  threadStart = _
    New System.Threading.ThreadStart( _
          AddressOf findNextHighestPrime)

  Dim newThread As System.Threading.Thread
  newThread = New System.Threading.Thread(threadStart)

  '  ,   
  setProcessingState(ProcessingState.waitingToStartAsync)
  newThread.Start()
End Sub

'------------------------------------------------------------------
'  .     
'      ,   
'    :
' (a)    
' (b)   (   )   
'      
'------------------------------------------------------------------
Public Sub findNextHighestPrime()
  '    ,   
  '  
  If (getProcessingState() = ProcessingState.requestAbort) Then
    GoTo finished_looking
  End If

  '  ,   
  setProcessingState(ProcessingState.lookingForPrime)

  m_startTickCount = System.Environment.TickCount
  Dim currentItem As Long
  ',    
  If ((m_startPoint And 1) = 1) Then
    '  ,      
    currentItem = m_startPoint + 2
  Else
    '  ,       
    currentItem = m_startPoint + 1
  End If

  '    
  While (getProcessingState() = ProcessingState.lookingForPrime)
    '      
    If (isItemPrime(currentItem) = True) Then
      m_NextHighestPrime = currentItem
      ' 
      setProcessingState(ProcessingState.foundPrime)
    End If

    currentItem = currentItem + 2
  End While

finished_looking:
    '.        
    '  ,     
    '   

    ' 
    m_endTickCount = System.Environment.TickCount

  '    , 
  ',    
  If (getProcessingState() = ProcessingState.requestAbort) Then
    setProcessingState(ProcessingState.aborted)
  End If
End Sub

' ,  , 
'  
Private Function isItemPrime(ByVal potentialPrime _
                                  As Long) As Boolean

  '  - , ,    
  If ((potentialPrime And 1) = 0) Then
    Return False
  End If

  '    ,      
  '   
  Dim end_point_of_search As Long
  end_point_of_search = _
      CLng(System.Math.Sqrt(potentialPrime) + 1)

  Dim current_test_item As Long = 3
  While (current_test_item <= end_point_of_search)

    '---------------------------------------------------------
    ',      !
    '---------------------------------------------------------
    If (getProcessingState() <> _
      ProcessingState.lookingForPrime) Then
      Return False
    End If

    '    , 
    ',    
    If (potentialPrime Mod current_test_item = 0) Then
      Return False
    End If

    '   
    current_test_item = current_test_item + 2
  End While

  '  
  Return True
End Function
End Class
